html, body {
  background-color: #eee;
}
body {
  padding-top: 40px; /* 40px to make the container go all the way to the bottom of the topbar */
}
.container > footer p {
  text-align: center; /* center align it with the container */
}
.container {
  width: 940px; /* downsize our container to make the content feel a bit tighter and more cohesive. NOTE: this removes two full columns from the grid, meaning you only go to 14 columns and not 16. */
}

/* The white background content wrapper */
.content {
  background-color: #fff;
  padding: 20px;
  margin: 0 -20px; /* negative indent the amount of the padding to maintain the grid system */
  -webkit-border-radius: 0 0 6px 6px;
     -moz-border-radius: 0 0 6px 6px;
          border-radius: 0 0 6px 6px;
  -webkit-box-shadow: 0 1px 2px rgba(0,0,0,.15);
     -moz-box-shadow: 0 1px 2px rgba(0,0,0,.15);
          box-shadow: 0 1px 2px rgba(0,0,0,.15);
}

/* modeladmin filters */
div#filter-wrapper {
  border: 1px solid #eee;
  border-radius: 3px;
  margin: 20px 0;
  padding: 5px 10px;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
}
form.modeladmin-filters { margin: 0; }
form.modeladmin-filters ul { list-style-type: none; }
form.modeladmin-filters select.field { margin-right: 10px; }

div.form-wrapper { width: 100%; overflow: auto; }

form textarea { height: 150px; }

div.sidebar li.active { font-weight: bold; }

/* helpers */
.hidden { display: none; }
