{% extends "admin/base.html" %}

{% block content_title %}Admin Dashboard{% endblock %}

{% block content %}
  {% if panels %}
    <div class="row">
    {% for panel in panels %}
      {{ panel.render()|safe() }}
    {% endfor %}
    </div>
  {% endif %}
{% endblock %}
